// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_VALUE_BASE_H
#define OB_VALUE_BASE_H

namespace CORBA
{

class ValueBase
{
    //
    // Not implemented
    //
    void operator=(const ValueBase&);

    static const char* _ob_id_;
    static const char* _ob_truncatable_ids_[];

protected:
    ValueBase();
    ValueBase(const ValueBase&);
    
    virtual ~ValueBase();

public:
    virtual void _add_ref() = 0;
    virtual void _remove_ref() = 0;
    virtual ULong _refcount_value() = 0;
    virtual ValueBase* _copy_value() = 0;

    virtual ValueDef_ptr _get_value_def()
        throw(SystemException);

    static ValueBase* _downcast(ValueBase*);

    static const char* _OB_id();
    virtual const char** _OB_truncatableIds() const;

    virtual const char* _OB_typeId() const;
    virtual Boolean _OB_isCustom() const;
    virtual Boolean _OB_isTruncatable() const;

    virtual void _OB_marshalValue(OB::OutputStream&);
    virtual void _OB_marshalCountValue(OB::CountStream&);
    virtual void _OB_unmarshalValue(OB::InputStream&);

    static void _OB_marshal(ValueBase*, OB::OutputStream&);
    static void _OB_marshalCount(ValueBase*, OB::CountStream&);
    static void _OB_unmarshal(ValueBase*&, OB::InputStream&);

    virtual AbstractBase_ptr _OB_toAbstractBase();
};

} // End of namespace CORBA

void OBRelease(CORBA::ValueBase*);
void OBDuplicate(CORBA::ValueBase*);

namespace CORBA
{

typedef OB::ObjVar< ValueBase > ValueBase_var;

inline void
add_ref(ValueBase* vb)
{
    if(vb != 0)
        vb -> _add_ref();
}

inline void
remove_ref(ValueBase* vb)
{
    if(vb != 0)
        vb -> _remove_ref();
}

class DefaultValueRefCountBase : public virtual ValueBase,
                                 public virtual OB::RefCount
{
private:
    //
    // Not implemented
    //
    void operator=(const DefaultValueRefCountBase&);

protected:
    DefaultValueRefCountBase();
    DefaultValueRefCountBase(const DefaultValueRefCountBase&);
    virtual ~DefaultValueRefCountBase();

public:
    virtual void _add_ref();
    virtual void _remove_ref();
    virtual ULong _refcount_value();
};

} // End of namespace CORBA

namespace PortableServer
{

//
// class PortableServer::ValueRefCountBase
//

class ValueRefCountBase : public virtual CORBA::ValueBase,
                          public virtual RefCountServantBase
{
private:
    //
    // Not implemented
    //
    void operator=(const ValueRefCountBase&);

protected:
    ValueRefCountBase();
    ValueRefCountBase(const ValueRefCountBase&);
    virtual ~ValueRefCountBase();

public:
    virtual void _add_ref();
    virtual void _remove_ref();
    virtual CORBA::ULong _refcount_value();
};

} // End of namespace PortableServer

#endif
